//-----------------------------------------------------------------------------
// XAPP715_top - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        XAPP715_top.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.1
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    ver1.0 Initial Release 
//   3/31/2005    STAM      ver1.1 removed FDDRCPE in clkrst.v
//-----------------------------------------------------------------------------

// This is the basic implementation of second order 5th dimension Reed-Muller code
// with the following properties : n=32, k=16, t=3, Hamming weight = 8
// Datawidth = 16, codeword=32, 7-stage pipeline
// Further details including operating principles can be found in XAPP715

`timescale 1ns/100ps
									
module XAPP715_top(clkin, reset, force_error, data_p,code_out_p,code_in_p,m,error, clkout, locked);
    input 			clkin;         //clock input
    input 			reset;         //DCM reset
    input  [2:0]	force_error;
    input  [15:0] data_p;	    // input data
    output [31:0] code_out_p;	//codeword output
    input  [31:0] code_in_p;	//codeword input
    output [15:0] m;  		   // decoded message
    output [1:0]	error;		//error status
    output        clkout;
    output 			locked;


wire        clkfb, clk2x, locked;
wire [7:0]  x9_s1, x8_s1, x7_s1, x6_s1, x5_s1, x4_s1, x3_s1, x2_s1, x1_s1, x0_s1;
wire [15:0] x14_s2, x13_s2, x12_s2, x11_s2, x10_s2;
wire [31:0] x15_s3;
wire [15:0] data;
wire [31:0] code_out, code_in, code_s2, code_s3;
wire [1:0]  error_s1_p, error_s2_p;
wire [9:0]  x_s1;
wire [14:10] x_s2;
wire	      x_s3;

reg [31:0]  code_s2_m2, code_s2_m3, code_s2_m4, code_s3_m5;
reg [1:0]   error_s1_m1, error_s1_m2, error_s1_m3, error_s1_m4;
reg [1:0]   error_s1_m5, error_s1_m6;
reg [15:0]  m1, x14_s2_m3, x13_s2_m3, x12_s2_m3, x11_s2_m3, x10_s2_m3, m3, m4, m5, m6;
reg [9:0]   m2;
reg [15:0]  data_reg;
reg [31:0]  code_out_reg, code_in_reg, code_in_s1;

// DCM and reset
clkrst u_clkrst (
		.clkin(clkin),
		.reset(reset),
		.clk2x(clk),
		.locked(locked)
		);
assign clkout = clk;

// pipelines
always @( posedge clk or posedge reset) begin
	if (reset) 
		begin
		data_reg <= 15'b0;
	 	code_out_reg <= 32'b0;
		code_in_reg <= 32'b0;
		code_in_s1 <= 32'b0;
		code_s2_m2 <= 32'b0;
		code_s2_m3 <= 32'b0;
		code_s2_m4 <= 32'b0;
		m1 <= 16'b0;
		m2 <= 16'b0;
		m3 <= 16'b0;
		m4 <= 16'b0;
		m5 <= 16'b0;
		end
	else
		begin	
      data_reg <= data_p;
	 	code_out_reg <= code_out;
		code_in_reg <= code_in_p;

		code_in_s1 <= code_in;
		m1[9:0] <=  x_s1;
		error_s1_m1 <= error_s1_p;

		m2[9:0] <= m1;
		code_s2_m2 <= code_s2;
		error_s1_m2 <= error_s1_m1;

		x14_s2_m3 <= x14_s2;
		x13_s2_m3 <= x13_s2;
		x12_s2_m3 <= x12_s2;
		x11_s2_m3 <= x11_s2;
		x10_s2_m3 <= x10_s2;
		m3[9:0] <= m2[9:0];
		code_s2_m3 <= code_s2_m2;
		error_s1_m3 <= error_s1_m2;

		m4[9:0] <= m3[9:0];
		m4[14:10] <= x_s2;
		code_s2_m4 <= code_s2_m3;
		error_s1_m4 <= error_s1_m3;

		code_s3_m5 <= code_s3;
		error_s1_m5 <= error_s1_m4;
		m5[14:0] <= m4[14:0];

		m6[14:0] <= m5[14:0];
		m6[15] <= x_s3;
		error_s1_m6 <= error_s1_m5;
		end
end

assign data = data_reg;
assign code_out_p = code_out_reg;
assign code_in = code_in_reg;
assign m = m6;
assign error = error_s1_m6;

//===============
//--- Encoder ---
//===============
encoder u_encoder (	
		.clk(clk),
		.reset(reset),
		.force_error(force_error), 
		.data(data),
		.code_out(code_out)
		);

//=======================
//--- Stage 1 decoder ---
//=======================
// decodes data[9:0]
//Block A
//orthogonal checksum generator
// It generates 8-bit CS for bit[9-0]

ocs_gen_s1 u_ocs_gen_s1 (
		.code_in(code_in),
		.x9_s1(x9_s1),
		.x8_s1(x8_s1),
		.x7_s1(x7_s1),
		.x6_s1(x6_s1),
		.x5_s1(x5_s1),
		.x4_s1(x4_s1),
		.x3_s1(x3_s1),
		.x2_s1(x2_s1),
		.x1_s1(x1_s1),
		.x0_s1(x0_s1)
		);

// Majority Logic decoder
// Block B

MLD_s1 u_MLD_s1 (
		.clk(clk),
		.reset(reset),
		.x9_s1(x9_s1),
		.x8_s1(x8_s1),
		.x7_s1(x7_s1),
		.x6_s1(x6_s1),
		.x5_s1(x5_s1),
		.x4_s1(x4_s1),
		.x3_s1(x3_s1),
		.x2_s1(x2_s1),
		.x1_s1(x1_s1),
		.x0_s1(x0_s1),
		.x(x_s1),
		.error(error_s1_p)
		);
 

//Block C
// generates partial codeword C'
code_gen_s2 u_code_gen_s2 (
		.data(m1[9:0]),
		.code_in(code_in_s1),
		.code_out(code_s2)
		);

//=======================
//--- Stage 2 decoder ---
//=======================
// decodes data[14:10]
// Block D
// orthogonal checksum generator
ocs_gen_s2 u_ocs_gen_s2 (
		.code_in(code_s2_m2),
		.x14_s2(x14_s2),
		.x13_s2(x13_s2),
		.x12_s2(x12_s2),
		.x11_s2(x11_s2),
		.x10_s2(x10_s2)
		);

// Block E
MLD_s2 u_MLD_s2 (
		.clk(clk),
		.reset(reset),
		.x14_s2(x14_s2_m3),
		.x13_s2(x13_s2_m3),
		.x12_s2(x12_s2_m3),
		.x11_s2(x11_s2_m3),
		.x10_s2(x10_s2_m3),
		.x(x_s2)
		);

//=======================
//--- Stage 3 decoder ---
//=======================
// decodes data[15]
// Block F
// generate partial codeword C''

code_gen_s3 u_code_gen_s3 (
		.datain(m4[14:10]),
		.code_in(code_s2_m4),
		.code_out(code_s3)
		);

//Block G
MLD_s3 u_MLD_s3 (
		.clk(clk),
		.reset(reset),
		.x15_s3(code_s3_m5),
		.x(x_s3)
		);

endmodule

 